unit uRC4;

{ In dieser Unit wird RC4/CipherSaber2 implementiert.
  Informationen zu CipherSaber2 lassen sich auf
  http://ciphersaber.gurus.com/ finden. Eine Beschreibung
  des Algorithmus befindet sich auf:
  http://ciphersaber.gurus.com/faq.html#getrc4
  Als n (fr CipherSaber2) wird 20 gewhlt. }

interface

type
  TByteBlock   = array [0..255] of Byte;
  TCipherSaber = class
    StateArray, KeyArray : TByteBlock;
    keylength            : Byte;
    datlength, cs2_n     : LongInt;
    ciphermode           : boolean;
    DataArray            : array of Byte;
    InitVector           : array of Byte;
    constructor ApplyCS2(Key, Data : string; encrypt : boolean; cs2_n : longint); // Ok, einen Konstruktor verwenden der gleich alles erledigt ist nicht sonderlich schn, aber in diesem Fall nicht unpraktisch.
    procedure SetupStateArray;
    procedure ApplyCipher;
    procedure SwapElements(var ByteBlock : TByteBlock; i, j : Byte);
    function GetData : string;
  end;

implementation

function TCipherSaber.GetData;
var
  datastring : string;
  i          : Longint;
begin
  datastring := ''; // Der String ist zunchst leer.
  if ciphermode then // Falls verschlsselt mit IV, sonst ohne.
    for i := 0 to 9 do // Jedes Byte des IV...
      begin
        datastring := datastring + chr(InitVector[i]); // ...wird an datastring angehngt.
        InitVector[i] := 0; // Wird auf 0 gesetzt.
      end;

  ciphermode := false; // Wird auf 0 gesetzt.

  for i := 0 to (datlength - 1) do // Jedes Byte des DataArray...
    begin
      datastring := datastring + chr(DataArray[i]); // ...wird an datastring angehngt.
      DataArray[i] := 0; // Wird auf 0 gesetzt.
    end;

  datlength := 0; // Wird auch auf 0 gesetzt.

  result := datastring; // datastring ist unser Rckgabewert.
end;

constructor TCipherSaber.ApplyCS2;
var
  i, j : LongInt;
begin
  self.cs2_n := cs2_n;
  ciphermode := encrypt; // Wir brauchen diese Angabe bei der Rckgabe der Daten, um festzustellen ob wir den IV vorne anhngen oder nicht.
  keylength := (Length(Key) mod 247) + 10; // Schlssellnge feststellen (max. 246 + 10 fr den "Initialization Vector").
  if encrypt then // Wird verschlsselt oder entschlsselt?;
    begin
      for i := 0 to 245 do // Die Restlichen Bytes im KeyArray:
        if i < (keylength - 10) then // Nicht ber den Schlssel hinauslaufen.
          begin
            KeyArray[i] := ord(Key[i + 1]); // Die Ordnungszahlen der Bytes des Schlsselstrings in den KeyArray eintragen.
            Key[i] := chr(0); // Byte des Schlssels auf 0 setzen.
          end;
      SetLength(InitVector, 10); // Wir haben einen IV, also setzen wir die Lnge auf 10.
      j := 0; // Auf 0 setzen.
      for i := (keylength - 10) to keylength do // Jedes Byte des IV:
        begin
          KeyArray[i] := random(256); // Mglicherweise wre ein anderer Zufallsgenerator "sicherer" als der Standard-Delphi-PRNG.
          InitVector[j] := KeyArray[i]; // IV speichern, um ihn spter vorne an die Daten anzuhngen.
          j := j + 1; // Um 1 erhhen. Dies ist die Zhlvariable fr den IV.
        end;
    end
  else
    begin
      for i := 0 to 245 do // Die Restlichen Bytes im KeyArray:
        if i < (keylength - 10) then // Nicht ber den Schlssel hinauslaufen.
          begin
            KeyArray[i] := ord(Key[i + 1]); // Die Ordnungszahlen der Bytes des Schlsselstrings in den KeyArray eintragen.
            Key[i + 1] := chr(0); // Byte des Schlssels auf 0 setzen.
          end;
      for i := (keylength - 10) to keylength - 1 do // Jedes Byte des IV:
        begin
          KeyArray[i] := ord(Data[i - (keylength - 11)]); // Die ersten 10 Elemente des KeyArray werden aus dem IV genommen, welcher die ersten 10 Byte der Daten sind.
          Data[i - (keylength - 11)] := chr(0); // Byte der Daten auf 0 setzen.
        end;
      Delete(Data, 1, 10); // Der IV gehrt nicht zu den Daten.
    end;
  SetupStateArray; // StateArray initialisieren.

  datlength := Length(Data); // Lnge der Daten feststellen.
  SetLength(DataArray, datlength); // Lnge von DataArray entsprechend setzen.
  for i := 0 to (datlength - 1) do
    DataArray[i] := ord(Data[i + 1]); // String zu TByteBlock umbauen.

  for i := 1 to datlength do
    Data[i] := chr(0); // Mglicherweise unverschlsselte Daten aus dem Speicher entfernen.
end;

procedure TCipherSaber.SetupStateArray; // Hier wird das StateArray initialisiert.
var
  i, j, n, k : Byte; // Indizes fr Arrays.
begin
  for i := 0 to 255 do // StateArray wird mit den Zahlen 0 bis 255 gefllt.
    StateArray[i] := i;

  j := 0;
  for k := 1 to cs2_n do // Wiederhole n Mal, fr CipherSaber2:
    for i := 0 to 255 do begin // Vermischung fr jedes der Elemente aus StateArray.
      n := i mod keylength; // n darf nicht hher werden als die Lnge des Schlssels.
      j := (j + StateArray[i] + KeyArray[n]) mod 256; // Zu j werden j, das i-te Element des StateArrays und das n-te Element des KeyArrays addiert, modulo 256 (es wird maximal 255).
      SwapElements(StateArray, i, j); // Das i-te und j-te Element des StateArrays werden vertauscht.
    end;

  for i := 0 to 255 do
    KeyArray[i] := 0; // Die Werte im Schlsselarray auf 0 setzen.
  keylength := 0;
  j := 0; // Auf 0 setzen.
  n := 0; // Auch auf 0 setzen.
  k := 0; // und nochmal.
end;

procedure TCipherSaber.ApplyCipher;
var
  c       : LongInt; // Zhler fr DataArray
  i, j, n : Byte; // Indizes fr Arrays.
begin
  i := 0;
  j := 0;
  for c := 0 to (datlength - 1) do begin // Fr jedes Byte im DataArray:
    i := (i + 1) mod 256; // i um 1 erhhen, aber unter 256 halten.
    j := (j + StateArray[i]) mod 256; // Das i-te Element des StateArrays wird zu j addiert, ebenfalls modulo 256.
    SwapElements(StateArray, i, j); // Vertauschen von i-tem und j-tem Element des StateArrays.
    n := (StateArray[i] + StateArray[j]) mod 256; // Durch Addition des i-ten und j-ten Elementes des StateArrays, modulo 256, erhalten wir n.
    DataArray[c] := StateArray[n] xor DataArray[c]; // Der Ausgabewert wird erzeugt, indem der Originalwert mit dem Wert des n-ten Elementes des StateArrays geXORt wird.
  end;

  for i := 0 to 255 do
    StateArray[i] := 0; // Die Werte im StateArray auf 0 setzen.
  j := 0; // Auf 0 setzen.
  n := 0; // Auch auf 0 setzen.
  n := 0; // und nochmal.
end;

procedure TCipherSaber.SwapElements;
var
  temp : Byte; // Fr einen Ringtausch wird eine zustzliche variable bentigt.
begin
  temp := ByteBlock[i];
  ByteBlock[i] := ByteBlock[j];
  ByteBlock[j] := temp;
  temp := 0; // Auf 0 setzen, um keine etwaigen Daten des Verschlsselungsprozesses im Speicher zu hinterlassen.
end;

end.
